"""
Description:
    A Module to log and store information about a Pokémon tcg collection
Usage:
    To run as a program "python3 main.py"
    To use as a library:
        "from pokemonCardLogger import clss as pcl" for json storage
        "from pokemonCardLogger import clss_alt as pcl" for sqlite storage (pre-alpha experimental)
    To get just the packs and their ids, run "python3 packRef.py"
"""
from pokemonCardLogger import *
